;===========================
;SOFTWARE FOR BLINKSTROEM V2
;===========================

;  RS232 stream modifications by Arne Rossius
;   *** all modifications are preceded by the line "modification for RS232 stream" ***
;   *** you cannot use the keys with this software! ***

; > start stream: remove card, send one valid frame
; > end stream: insert card

;stream data format: MCUF

;neccesary hardware modifications: - disconnect PD0 from ROW7
;                                  - reconnect ROW7 to PB3
;                                  - connect PD0 to a RS232 level converter


;  Ver. S-0.1   2005-10-14   Arne Rossius

			.include "m16def.inc"
			
			
			
			.def	akku		= r21
			.def	col0		= r8
			.def	col1		= r9
			.def	col2		= r10
			.def	colnr		= r11
			
			.def	framecnt	= r12
			
			;Bit 0=1 => Card has just been inserted
			;Bit 1=1 => Card has been removed
			;Bit 2=1 => Card is inserted
			;modification for RS232 stream: Bit 3=1 => Stream is running
			.def	statusreg	= r13
			
			.def	dur0		= r15
			.def	dur1		= r25
			
			;modification for RS232 stream:
			.def	bufpos = R22
			.def	timeout = R14	
			
		;moved here from SdcFat16.asm
		.def	arg0 = r1
		.def	arg1 = r2
		.def	arg2 = r3
		.def	arg3 = r4		;Arguments for SDC-commands
		.def	temp = r16
		.def	cnt0 = r17
		.def	cnt1 = r18
		.def	cnt2 = r19
		.def	cnt3 = r20

			
			.cseg
			.org 0
			rjmp	Reset
			
			.org	OVF0addr
			;modification for RS232 stream:
			rjmp	timer_overflow
			.org	URXCaddr
			rjmp	rs232_rx
			
			
			
;===============================================================================
			
;modification for RS232 stream (whole interrupt routine):
rs232_rx:
			push	temp
			in	temp, SREG
			push	temp
			push	ZL
			push	ZH
			push	cnt0
			push	akku
			push	cnt1
			push	cnt2
			
			;reset timeout
			clr	timeout
			
			;save byte to RAM
			in	temp, UDR
			ldi	ZL, LOW(RS232Buffer)
			ldi	ZH, HIGH(RS232Buffer)
			add	ZL, bufpos
			brcc	PC+2
				inc	ZH
			st	Z, temp
			inc	bufpos
			
			;check whether the packet is complete:
			cpi	bufpos, 156
			brsh	rs232_eval
			
rs232_ret:		;return
			pop	cnt2
			pop	cnt1
			pop 	akku
			pop 	cnt0
			pop 	ZH
			pop 	ZL
			pop 	temp
			out	SREG, temp
			pop	temp
			reti

rs232_err:		;error: throw away the packet
			ldi	bufpos, 0
			rjmp	rs232_ret
			
			
			
rs232_eval:	;evaluate packet
			;evaluate packet header:
			ldi	ZL, LOW(RS232Buffer)
			ldi	ZH, HIGH(RS232Buffer)
			ld	temp, Z+ ;magic = 0x23 0x54 0x26 0x66
			cpi	temp, 0x23
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x54
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x26
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x66
			brne	rs232_err
			ld	temp, Z+ ;height = 0x00 0x08 (8)
			cpi	temp, 0x00 
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x08
			brne	rs232_err
			ld	temp, Z+ ;width = 0x00 0x12 (18)
			cpi	temp, 0x00
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x12
			brne	rs232_err
			ld	temp, Z+ ;channels = 0x00 0x01 (1)
			cpi	temp, 0x00
			brne	rs232_err
			ld	temp, Z+
			cpi	temp, 0x01
			brne	rs232_err
			ld	temp, Z+ ;maxval = 0x00 0x00..0xFF (0..255)
			cpi	temp, 0x00
			brne	rs232_err
			ld	akku, Z+
			
			;enable stream if no card is inserted
			sbrc	statusreg, 2
			rjmp	rs232_ret
			ldi	temp, 0x08
			or	statusreg, temp
			
			;copy packet data to current frame RAM and convert from [0..akku] to [0..7]
			ldi	cnt0, 0
			ldi	cnt1, 143
rs232_copy_read:
			ldi	ZL, LOW(RS232Buffer+12)
			ldi	ZH, HIGH(RS232Buffer+12)
			add	ZL, cnt0
			brcc	PC+2
				inc	ZH
			ld	temp, Z
			
			mov	cnt2, akku
			cpi	cnt2, 7
			brlo	rs232_copy_convert_up
rs232_copy_convert_down:
			cpi	cnt2, 8
			brlo	rs232_copy_write
			lsr	temp
			lsr	cnt2
			rjmp	rs232_copy_convert_down
			
rs232_copy_convert_up:
			lsl	temp
			cp	temp, cnt2
			brlo	PC+2
			inc	temp
			sec
			rol	cnt2
			cpi	cnt2, 7
			brlo	rs232_copy_convert_up
			
rs232_copy_write:
			ldi	ZL, LOW(Row0)
			ldi	ZH, HIGH(Row0)
			add	ZL, cnt1
			brcc	PC+2
				inc	ZH
			st	Z, temp
			subi	cnt1, 8 ;turn frame 90' and mirror to use it with BSA
			brcc	PC+3
				ldi	temp, 143
				add	cnt1, temp
			inc	cnt0
			cpi	cnt0, 144
			brlo	rs232_copy_read
			ldi	bufpos, 0
			rjmp	rs232_ret

;===============================================================================
			
			
			
timer_overflow:
			push	temp
			in	temp, sreg
			push	temp		;save registers on stack
			push	zl
			push	zh
			push	cnt0
			push	akku
			push	cnt1
			
			;modification for RS232 stream: timeout
			inc	timeout
			brne	timeout_end
			ldi	bufpos, 0
timeout_end:
			
			ldi	temp, 256-28 	;700sec pro Zeile
			out	tcnt0, temp
			
			;Check availability of mm- or sd card
			mov	temp, statusreg
			sbic	pinb, 2
			rjmp	nocard
				sbrc	statusreg, 2
				rjmp	yescard
					ori	temp, 4
					ori	temp, 1
			rjmp	yescard
nocard:				sbrs	statusreg, 2
				rjmp	yescard
					andi	temp, ~4
					ori	temp, 2
					rcall	SDC_Release	;dafr sorgen, dass Karte spannungsfrei geschaltet wird!
yescard:		mov 	statusreg, temp

			in	temp, portb		;Ja, PORTb ist richtig!
				andi	temp, 0b11111100
				mov	cnt1, col2
				andi	cnt1, 0b00000011
				or	temp, cnt1
			out	portc, col0
			out	porta, col1
			out	portb, temp			
			
			rcall	initline	;display a single column
			
			clr	temp
			out	portd, temp
			;modification for RS232 stream:
			cbi	PORTB, 3
			
			sec
			rol	col0
			ror	col1
			rol	col2
			
			inc	colnr
			ldi	temp, 18
			cp	colnr, temp
			brne	OvF0End
				;Hier: 1 Bild komplett angezeigt
				ldi	temp, $fe
				mov	col0, temp
				ldi	temp, $ff
				mov	col1, temp
				mov	col2, temp
				clr	colnr
				inc	framecnt
OvF0End:		pop	cnt1
			pop	akku
			pop	cnt0
			pop	zh
			pop	zl
			pop	temp
			out	sreg, temp
			pop	temp
			reti
			
			
			.include "SdcFat16.asm"
			.include "font.asm"
			.cseg
			
			
Reset:			ldi	temp, low(RAMEND)
			out	spl, temp
			ldi	temp, high(RAMEND)
			out	sph, temp
			
			ldi	temp, $00
			out	portd, temp
			ldi	temp, $ff
			out	ddrd, temp		;portd = ROW drivers
			
			ldi	temp, $ff
			out	porta, temp
			out	portc, temp
			out	ddra, temp
			out	ddrc, temp
			
			ldi	temp, $03
			out	ddrb, temp
			ldi	temp, $07
			out	portb, temp
			;modification for RS232 stream:
			sbi	DDRB, 3
			cbi	PORTB, 3
			
			ldi	akku, 0
			rcall	fillscreen 
			
			ldi	temp, $ff
			mov	col0, temp
			mov	col1, temp
			mov	col2, temp
			ldi	temp, 17
			mov	colnr, temp		;Screen anfangs schwarz!
			
			clr	statusreg
			
			;modification for RS232 stream: RS232 setup
			.equ	BAUD = 16000000/(8*57600)-1 ;57600 Baud @ 16 MHz with double-speed
			ldi	bufpos, 0
			ldi	temp, LOW(BAUD)
			out	UBRRL, temp
			ldi	temp, HIGH(BAUD)
			out	UBRRH, temp
			ldi	temp, 1<<U2X
			out	UCSRA, temp
			ldi	temp, 1<<RXEN | 1<<RXCIE
			out	UCSRB, temp
			
			;timer interrupt initialisieren
			ldi	temp, 0b100 		;ck = 7.372 MHz/256
			out	tccr0, temp
			ldi	temp, 0x01
			out	timsk, temp		;timer0 ovf int enable
			sei
			
			
			
			
			
WaitForCard:		sbis	pinb, 2			; skip "Wait for card" if card is already inserted
			rjmp	FatPartOfInit
			
			;modification for RS232 stream:
			sbrc	statusreg, 3		;no message display if stream is running
			rjmp	WaitForCard

			ldi	zl, low(StringInsertCard*2)
			ldi	zh, high(StringInsertCard*2)
			rcall	writecodestring
			sbrs	statusreg, 0 	;Card inserted
			rjmp	WaitForCard
			


			;Here: Card is inserted!
			mov	temp, statusreg
			andi	temp, 0xfe
			;modification for RS232 stream: disable stream
			andi	temp, ~0x08
			
			mov	statusreg, temp
				
			;ldi	zl, low(StringCardInserted*2)
			;ldi	zh, high(StringCardInserted*2)
			;rcall	writecodestring


FatPartOfInit:		rcall	FAT_init
			rcall	FAT_FindFirst
			; Verzeichnis 'BS2' suchen

readnextentry:		rcall	FAT_FindNext
			lds	akku, fatdirentry
			cpi	akku, 0x00
			brne	bs2found
			rjmp	bs2notfound
bs2found:			lds	akku, fatdirentry
				cpi	akku, 'B'
				brne	readnextentry
				lds	akku, fatdirentry+1
				cpi	akku, 'S'
				brne	readnextentry
				lds	akku, fatdirentry+2
				cpi	akku, '2'
				brne	readnextentry
			rcall	FAT_Findclose
					
			;Hier: Verzeichnis 'BS2' gefunden!
			;ldi	zl, low(StringFoundDir*2)
			;ldi	zh, high(StringFoundDir*2)
			;rcall	writecodestring			; Ich denke, wenn er anfngt zu spielen ist jedem klar, 
								; dass er die Datei gefunden hat.
			

			rcall	FAT_ReadStart
					
			FAT_LOOP:	rcall	FAT_ReadByte
					mov	dur0, temp
					rcall	FAT_ReadByte
					mov	dur1, temp	; "Dauer" des frames lesen - 2 Byte (1 Word)
					
					rcall	readframe	; Kompletten frame in framebuffer lesen - 144 Byte
					brcc	WaitForCardJumpHelper	; Carry gelscht => Dateiende erreicht oder Fehler aufgetreten
					
					;sbis	pinb, 1		; Wenn S1 gedrckt ist => Vorspulen!
					;rjmp	FAT_LOOP_NO
					
					clr	framecnt
			FAT_LOOP0:	cp	framecnt, dur0
					brne	FAT_LOOP0       ; Angegebene Anzahl von Frames warten
					

					clr	framecnt
			FAT_LOOP1:	cp	framecnt, dur0
					brne	FAT_LOOP1       ; Angegebene Anzahl von Frames warten
					
					rjmp 	FAT_LOOP	; Angegebene Dauer (=Anzahl der Frames) abwarten		
					
FAT_LOOP_NO:				clr	dur0
					inc	dur0
					clr	framecnt
			FAT_LOOP2:	cp	framecnt, dur0
					brne	FAT_LOOP2       ; Angegebene Anzahl von Frames warten
					rjmp	FAT_LOOP
			
			
WaitForCardJumpHelper:	
			;modification for RS232 stream: disable stream
			ldi	temp, ~0x08
			and	statusreg, temp			
			
			rjmp	WaitForCard
				
				
			
			
		
bs2notfound:		ldi	zl, low(StringDirNotFound*2)
			ldi	zh, high(StringDirNotFound*2)
			rcall	writecodestring
			rjmp	bs2notfound
			
			
			
			
			
			
			
readframe:		;ldi	ZL, low(Frames*2)
			;ldi	ZH, high(Frames*2)
			ldi	XL, low(Row0+144)
			ldi	XH, High(Row0+144)
			ldi	cnt0, 144
readframe0:		;lpm	temp, Z+
			push 	cnt0
			rcall	FAT_ReadByte
			pop	cnt0
			
			brcc    readframe1		; Carry gelscht => Fehler aufgetreten oder Dateiende erreicht
			;ldi	temp, 7
			
			st	-X, temp
			dec	cnt0
			brne	readframe0			
			sec				; Carry setzen, da alles ok ist
readframe1:		ret

			
			
			
			
;******************************************************************************
;** Display-Funktionen
;******************************************************************************			

;Fllt den Bildschirm mit dem in AKKU enthaltenen Farbwert
fillscreen:		ldi	XL, low(Row0)
			ldi	XH, high(Row0)
			ldi	cnt0, 144
fillscreen0:		st	X+, akku
			dec	cnt0
			brne	fillscreen0
			ret
			
;Gibt den String, auf den Z zeigt als Laufschrift aus
writecodestring:	lpm	akku, Z+
			cpi	akku, 0
			breq	writecodestring0
			push	zl
			push	zh
			rcall   writechar
			pop	zh
			pop	zl
			rjmp	writecodestring
writecodestring0:	ldi	akku, ' '
			rcall	writechar		
			ldi	akku, ' '
			rcall	writechar
			ldi	akku, ' '
			
;Gibt ein einzelnes Zeichen (aus Register AKKU) als Laufschrift aus
writechar:		
			subi	akku, $20
			lsl	akku		;akku = akku*2
			ldi	temp, 0
			ldi	ZL, low(fonttable*2)
			ldi	ZH, high(fonttable*2)
			add	ZL, akku
			adc	ZH, temp
			add	ZL, akku
			adc	ZH, temp
			add	ZL, akku
			adc	ZH, temp
			add	ZL, akku
			adc	ZH, temp
			ldi	cnt1, 8		;8 Spalten = 1 Zeichen
writechar0:		rcall	moveleft	;Screeninhalt 1 Pixel nach links schieben
			lpm	temp, Z+
			ldi	XL, low(Row0)
			ldi	XH, high(Row0)
			ldi	cnt0, 128
writechar1:		ldi	cnt2, 0
			mov	akku, temp
			and	akku, cnt0
			tst	akku
			breq	writechar2	;Bit is "Eins"
				ldi	cnt2, 7
writechar2:		st	X+, cnt2
			lsr	cnt0
			tst	cnt0
			brne	writechar1
			clr	framecnt
writechar3:		ldi	temp, 3
			cp	framecnt, temp
			brne	writechar3
			dec	cnt1
			brne	writechar0
			ret
			
			
			
			
;Scrollt den Screeninhalt um einen Pixel nach links
moveleft:		ldi	YL, low(rowG+8)
			ldi	YH, high(rowG+8)
			ldi	XL, low(rowH+8)
			ldi	XH, high(rowH+8)
			ldi	cnt0, 144-8
moveleft0:		ld	temp, -Y
			st	-X, temp
			dec	cnt0
			brne	moveleft0
			ret
			
			

			

			
			
;******************************************************************************
;** Low-level kram fr Displayupdate (wird vom Timer0 OVF Int aufgerufen			
;******************************************************************************			
			
initline:		ldi	ZL, low(RowTable*2)
			ldi	ZH, high(RowTable*2)
				mov	temp, colnr
				add	ZL, temp
				ldi	temp, 0
				adc	ZH, temp
				mov	temp, colnr
				add	ZL, temp
				ldi	temp, 0
				adc	ZH, temp
			lpm	temp, Z+
			lpm	ZL, Z+
			mov	ZH, temp
			
			ld	temp, Z+
			sts	CurRow+0, temp
			ld	temp, Z+
			sts	CurRow+1, temp
			ld	temp, Z+
			sts	CurRow+2, temp
			ld	temp, Z+
			sts	CurRow+3, temp
			ld	temp, Z+
			sts	CurRow+4, temp
			ld	temp, Z+
			sts	CurRow+5, temp
			ld	temp, Z+
			sts	CurRow+6, temp
			ld	temp, Z+
			sts	CurRow+7, temp			
			.macro	decpixel
						ld	akku, -Z
						dec	akku
						brmi	decpixel0
							st	Z, akku
							ori	temp, @0		;7/5 zyklen
			decpixel0:
			.endm
showline:		clr	cnt1
showlineB:		ldi	ZL, low(CurRow+8)
			ldi	ZH, high(CurRow+8)
			clr	temp
			decpixel 1
			decpixel 2
			decpixel 4
			decpixel 8
			decpixel 16
			decpixel 32
			decpixel 64
			decpixel 128
			out	portd, temp
			;modification for RS232 stream:
			cbi	PORTB, 3
			sbrc	temp, 0
			sbi	PORTB, 3

			inc	cnt1
			mov	akku, cnt1
showline8:		dec	akku
			breq	showlinea
			ldi	temp, 50
showline9:		dec	temp
			brne	showline9
			rjmp	showline8
showlinea:		
			cpi	cnt1, 8
			brne	showlineB
			ret
			
			
			
			
			
			
			
Rowtable:		.db	high(Row0), low(Row0)
			.db	high(Row1), low(Row1)
			.db	high(Row2), low(Row2)
			.db	high(Row3), low(Row3)
			.db	high(Row4), low(Row4)
			.db	high(Row5), low(Row5)
			.db	high(Row6), low(Row6)
			.db	high(Row7), low(Row7)
			.db	high(Row8), low(Row8)
			.db	high(Row9), low(Row9)
			.db	high(Rowa), low(Rowa)
			.db	high(Rowb), low(Rowb)
			.db	high(Rowc), low(Rowc)
			.db	high(Rowd), low(Rowd)
			.db	high(Rowe), low(Rowe)
			.db	high(Rowf), low(Rowf)
			.db	high(Rowg), low(Rowg)
			.db	high(Rowh), low(Rowh)
			
			
StringInsertCard:	.db "INSERT CARD",0x00

StringCardInserted:	.db "INSERTED", 0x00, 0x00

StringCardRemoved:	.db "CARD REMOVED", 0x00, 0x00

StringDirNotFound:	.db "FILE 'BS2' NOT FOUND", 0x00, 0x00

StringFoundDir:		.db "FOUND FILE BS2", 0x00, 0x00

;modification for RS232 stream:
StringRS232Error:	.db "RS232 ERROR", 0x00
StringRS232Timeout:	.db "RS232 TIMEOUT", 0x00
StringRS232:		.db "  RS232", 0x00
			
			
			
			.dseg
CurRow:			.BYTE	8
Row0:			.BYTE	8
Row1:			.BYTE	8
Row2:			.BYTE	8
Row3:			.BYTE	8
Row4:			.BYTE	8
Row5:			.BYTE	8
Row6:			.BYTE	8
Row7:			.BYTE	8
Row8:			.BYTE	8
Row9:			.BYTE	8
Rowa:			.BYTE	8
Rowb:			.BYTE	8
Rowc:			.BYTE	8
Rowd:			.BYTE	8
Rowe:			.BYTE	8
Rowf:			.BYTE	8
Rowg:			.BYTE	8
Rowh:			.BYTE	8
CurColNr:		.BYTE	1
CurColBm:		.BYTE	3
;modification for RS232 stream:
RS232Buffer:		.BYTE	156 ;12 byte header + 144 byte data